
package cn.sh.stc.sict.cloud.auth.handler;

import cn.sh.stc.sict.cloud.common.core.constant.RedisCacheConstant;
import cn.sh.stc.sict.cloud.common.security.config.UserAccountProperties;
import cn.sh.stc.sict.cloud.common.security.handler.AuthenticationFailureHandler;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.concurrent.TimeUnit;

/**
 *
 * 登录操作次数锁定功能
 */
@Slf4j
@Component
@AllArgsConstructor
public class SictAuthenticationFailureLockEventHandler implements AuthenticationFailureHandler {

	private final RedisTemplate<String, String> redisTemplate;
	private final UserAccountProperties userAccountProperties;

	/**
	 * 密码错误超过 5 此自动锁定
	 * <p>
	 * @param authenticationException 登录的authentication 对象
	 * @param authentication 登录的authenticationException 对象
	 * @param request 请求
	 * @param response 响应
	 */
	@Async
	@Override
	@SneakyThrows
	public void handle(AuthenticationException authenticationException, Authentication authentication,
			HttpServletRequest request, HttpServletResponse response) {
		// 只处理账号密码错误异常
		if (!(authenticationException instanceof BadCredentialsException) && userAccountProperties.getLimitFlag()) {
			return;
		}

		String username = authentication.getName();
		String key = String.format("%s:%s", RedisCacheConstant.LOGIN_ERROR_TIMES,  username);
		Long times = redisTemplate.opsForValue().increment(key);
		// 自动过期时间
		redisTemplate.expire(key, userAccountProperties.getLimitTime(), TimeUnit.MINUTES);
	}
}
